/***************************************************************************
*            file-src.c
*
*  jeu jun  2 18:51:16 2005
*  Copyright  2005  Philippe Rouquier
*  bonfire-app@wanadoo.fr
****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */




#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef BUILD_GDL

#include <glib.h>
#include <glib/gi18n-lib.h>

#include <gtk/gtkvpaned.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtkfilechooser.h>
#include <gtk/gtkfilechooserwidget.h>
#include <gtk/gtkfilefilter.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkuimanager.h>
#include <gtk/gtktoggleaction.h>
#include <gtk/gtkaction.h>

#include <gconf/gconf-client.h>

#include "file-src.h"

#ifdef BUILD_SEARCH
#include "search.h"
#endif

#ifdef BUILD_PLAYLIST
#include "play-list.h"
#endif

#ifdef BUILD_PREVIEW
#include "player.h"
#endif

static void bonfire_file_source_class_init (BonfireFileSourceClass *klass);
static void bonfire_file_source_init (BonfireFileSource * sp);
static void bonfire_file_source_finalize (GObject * object);

static void bonfire_file_source_switch_page_cb (GtkNotebook *notebook,
						GtkNotebookPage *page,
						guint page_num,
						BonfireFileSource *source);
static void bonfire_file_source_chooser_selection_changed_cb (GtkFileChooser *chooser,
							      BonfireFileSource *source);
static void bonfire_file_source_chooser_activated_cb (GtkFileChooser *chooser,
						      BonfireFileSource *source);
static void bonfire_file_source_display_chooser_changed_cb (GConfClient *client,
							    guint cxn,
							    GConfEntry *entry,
							    BonfireFileSource *source);
static void bonfire_file_source_chooser_toggled_cb (GtkToggleAction *action,
						    BonfireFileSource *source);
static void bonfire_file_source_show_chooser (BonfireFileSource *source);

#ifdef BUILD_SEARCH
static void bonfire_file_source_search_activated_cb (BonfireSearch *search,
						     BonfireFileSource *source);
static void bonfire_file_source_search_selection_changed_cb (BonfireSearch* search,
							     BonfireFileSource* source);
static void bonfire_file_source_display_search_changed_cb (GConfClient *client,
							   guint cxn,
							   GConfEntry *entry,
							   BonfireFileSource *source);
static void bonfire_file_source_search_toggled_cb (GtkToggleAction *action,
						   BonfireFileSource *source);
static void bonfire_file_source_show_search (BonfireFileSource *source);
#endif

#ifdef BUILD_PLAYLIST
static void bonfire_file_source_playlist_activated_cb (BonfirePlaylist *playlist,
						       BonfireFileSource *source);
static void bonfire_file_source_playlist_selection_changed_cb (BonfirePlaylist *playlist,
							       BonfireFileSource *source);

static void bonfire_file_source_display_playlist_changed_cb (GConfClient *client,
							     guint cxn,
							     GConfEntry *entry,
							     BonfireFileSource* source);
static void bonfire_file_source_playlist_toggled_cb (GtkToggleAction *action,
						     BonfireFileSource *source);
static void bonfire_file_source_show_playlist (BonfireFileSource *source);
#endif

#ifdef BUILD_PREVIEW
static void bonfire_file_source_display_viewer_changed_cb (GConfClient *
							   client,
							   guint cxn,
							   GConfEntry *
							   entry,
							   BonfireFileSource
							   * source);
static void bonfire_file_source_viewer_toggled_cb (GtkToggleAction *action,
						   BonfireFileSource *source);
static void bonfire_file_source_show_viewer (BonfireFileSource *source);
#endif

struct BonfireFileSourcePrivate {
	GtkWidget *notebook;
	GConfClient *client;

	GtkWidget *chooser;
	guint chooser_notify;

#ifdef BUILD_SEARCH
	GtkWidget *search;
	guint search_notify;
#endif

#ifdef BUILD_PLAYLIST
	GtkWidget *playlist;
	guint playlist_notify;
#endif

#ifdef BUILD_PREVIEW
	GtkWidget *viewer;
	guint viewer_notify;
#endif

	GtkActionGroup *action_group;
};

#define BONFIRE_DISPLAY_DIR	"/apps/bonfire/display"


static GtkActionEntry entries[] = {
	{"ViewMenu", NULL, "View"},
};

static GtkToggleActionEntry toggle_entries[] = {
	{"Chooser", NULL, N_("File chooser"), "F8", N_("Display file chooser"),
	 G_CALLBACK (bonfire_file_source_chooser_toggled_cb), FALSE},

#ifdef BUILD_SEARCH
	{"Search", NULL, N_("Search"), "F9", N_("Display search"),
	 G_CALLBACK (bonfire_file_source_search_toggled_cb), FALSE},
#endif

#ifdef BUILD_PLAYLIST
	{"Playlist", NULL, N_("Playlist"), "F10", N_("Display playlist"),
	 G_CALLBACK (bonfire_file_source_playlist_toggled_cb), FALSE},
#endif

#ifdef BUILD_PREVIEW
	{"Viewer", NULL, N_("Viewer"), "F7", N_("Display file viewer"),
	 G_CALLBACK (bonfire_file_source_viewer_toggled_cb), FALSE},
#endif
};

static const char *description = {
	"<ui>"
	    "<menubar name='menubar' >"
		"<menu action='ViewMenu'>"
			"<placeholder name='ViewPlaceholder'/>"

#ifdef BUILD_PREVIEW
			"<menuitem action='Viewer'/>"
#endif

			"<menuitem action='Chooser'/>"

#ifdef BUILD_SEARCH
			"<menuitem action='Search'/>"
#endif

#ifdef BUILD_PLAYLIST
			"<menuitem action='Playlist'/>"
#endif

			"<separator/>"
		"</menu>"
	    "</menubar>"
	"</ui>"
};

typedef enum {
	SELECTION_CHANGED_SIGNAL,
	ACTIVATED_SIGNAL,
	LAST_SIGNAL
} BonfireFileSourceSignalType;


static guint bonfire_file_source_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

GType
bonfire_file_source_get_type ()
{
	static GType type = 0;

	if (type == 0) {
		static const GTypeInfo our_info = {
			sizeof (BonfireFileSourceClass),
			NULL,
			NULL,
			(GClassInitFunc) bonfire_file_source_class_init,
			NULL,
			NULL,
			sizeof (BonfireFileSource),
			0,
			(GInstanceInitFunc) bonfire_file_source_init,
		};

		type = g_type_register_static (GTK_TYPE_VPANED,
					       "BonfireFileSource",
					       &our_info, 0);
	}

	return type;
}

static void
bonfire_file_source_class_init (BonfireFileSourceClass * klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);
	object_class->finalize = bonfire_file_source_finalize;

	/* signals */
	bonfire_file_source_signals[ACTIVATED_SIGNAL] =
	    g_signal_new ("activated", G_OBJECT_CLASS_TYPE (object_class),
			  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			  G_STRUCT_OFFSET (BonfireFileSourceClass,
					   activated), NULL, NULL,
			  g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

	bonfire_file_source_signals[SELECTION_CHANGED_SIGNAL] =
	    g_signal_new ("selection_changed",
			  G_OBJECT_CLASS_TYPE (object_class),
			  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
			  G_STRUCT_OFFSET (BonfireFileSourceClass,
					   selection_changed), NULL, NULL,
			  g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE,
			  1,
			  G_TYPE_BOOLEAN);
}

static void
bonfire_file_source_init (BonfireFileSource * obj)
{

	/* Set up all container */
	obj->priv = g_new0 (BonfireFileSourcePrivate, 1);

	obj->priv->client = gconf_client_get_default ();
	gconf_client_add_dir (obj->priv->client, "/apps/bonfire/display",
			      GCONF_CLIENT_PRELOAD_NONE, NULL);
	/* menus */
	obj->priv->action_group =
	    gtk_action_group_new ("FileSourceActions");
	gtk_action_group_set_translation_domain (obj->priv->action_group,
						 NULL);
	gtk_action_group_add_actions (obj->priv->action_group, entries,
				      G_N_ELEMENTS (entries), obj);
	gtk_action_group_add_toggle_actions (obj->priv->action_group,
					     toggle_entries,
					     G_N_ELEMENTS (toggle_entries),
					     obj);
	gtk_action_group_set_sensitive (obj->priv->action_group, FALSE);

	obj->priv->notebook = gtk_notebook_new ();
	gtk_notebook_set_show_border (GTK_NOTEBOOK (obj->priv->notebook),
				      TRUE);
	g_signal_connect (G_OBJECT (obj->priv->notebook),
			  "switch-page",
			  G_CALLBACK (bonfire_file_source_switch_page_cb),
			  obj);

	gtk_widget_show (obj->priv->notebook);
	gtk_paned_pack1 (GTK_PANED (obj), obj->priv->notebook, TRUE, FALSE);

#ifdef BUILD_PREVIEW
	bonfire_file_source_show_viewer (obj);

	if (gconf_client_get_bool (obj->priv->client, BONFIRE_KEY_DISPLAY_VIEWER, NULL) == TRUE) {
		GtkToggleAction *action;

		gtk_widget_show_all (obj->priv->viewer);
		action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
					    (obj->priv->action_group,
					    "Viewer"));
		gtk_toggle_action_set_active (action, TRUE);
	}

	obj->priv->viewer_notify = gconf_client_notify_add (obj->priv->client,
							    BONFIRE_KEY_DISPLAY_VIEWER,
							    (GConfClientNotifyFunc) bonfire_file_source_display_viewer_changed_cb,
							    obj, NULL, NULL);
#endif

#ifdef BUILD_PLAYLIST
	if (gconf_client_get_bool (obj->priv->client, BONFIRE_KEY_DISPLAY_PLAYLIST, NULL) == TRUE) {
		GtkToggleAction *action;

		bonfire_file_source_show_playlist (obj);
		action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
					    (obj->priv->action_group,
					    "Playlist"));
		gtk_toggle_action_set_active (action, TRUE);
	}

	obj->priv->playlist_notify =
	    gconf_client_notify_add (obj->priv->client,
				     BONFIRE_KEY_DISPLAY_PLAYLIST,
				     (GConfClientNotifyFunc) bonfire_file_source_display_playlist_changed_cb,
				     obj, NULL, NULL);
#endif

#ifdef BUILD_SEARCH
	if (gconf_client_get_bool (obj->priv->client, BONFIRE_KEY_DISPLAY_SEARCH, NULL) == TRUE) {
		GtkToggleAction *action;

		bonfire_file_source_show_search (obj);
		action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
					    (obj->priv->action_group,
					    "Search"));
		gtk_toggle_action_set_active (action, TRUE);
	}

	obj->priv->search_notify =
	    gconf_client_notify_add (obj->priv->client,
				     BONFIRE_KEY_DISPLAY_SEARCH,
				     (GConfClientNotifyFunc) bonfire_file_source_display_search_changed_cb,
				     obj, NULL, NULL);
#endif

	if (gconf_client_get_bool (obj->priv->client, BONFIRE_KEY_DISPLAY_CHOOSER, NULL) == TRUE) {
		GtkToggleAction *action;

		bonfire_file_source_show_chooser (obj);
		action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
					    (obj->priv->action_group,
					    "Chooser"));
		gtk_toggle_action_set_active (action, TRUE);
	}

	obj->priv->chooser_notify =
	    gconf_client_notify_add (obj->priv->client,
				     BONFIRE_KEY_DISPLAY_CHOOSER,
				     (GConfClientNotifyFunc)
				     bonfire_file_source_display_chooser_changed_cb,
				     obj, NULL, NULL);

	gtk_notebook_set_current_page (GTK_NOTEBOOK (obj->priv->notebook),
				       0);

	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (obj->priv->notebook)) == 1)
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (obj->priv->notebook),
					    FALSE);
}

static void
bonfire_file_source_show_chooser (BonfireFileSource *source)
{
	GtkWidget *label;
	GtkFileFilter *filter;

	source->priv->chooser = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (source->priv->chooser),
					      TRUE);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("All files"));
	gtk_file_filter_add_pattern (filter, "*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (source->priv->chooser),
				     filter);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("Audio files only"));
	gtk_file_filter_add_mime_type (filter, "audio/*");
	gtk_file_filter_add_mime_type (filter, "application/ogg");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (source->priv->chooser),
				     filter);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("Movies only"));
	gtk_file_filter_add_mime_type (filter, "video/*");
	gtk_file_filter_add_mime_type (filter, "application/ogg");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (source->priv->chooser),
				     filter);

	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("Image files only"));
	gtk_file_filter_add_mime_type (filter, "image/*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (source->priv->chooser),
				     filter);

	g_signal_connect (G_OBJECT (source->priv->chooser),
			  "file_activated",
			  G_CALLBACK (bonfire_file_source_chooser_activated_cb),
			  source);
	g_signal_connect (G_OBJECT (source->priv->chooser),
			  "selection_changed",
			  G_CALLBACK (bonfire_file_source_chooser_selection_changed_cb),
			  source);

	label = gtk_label_new (_("Files"));
	gtk_notebook_insert_page (GTK_NOTEBOOK (source->priv->notebook),
				  source->priv->chooser,
				  label,
				  0);
	gtk_widget_show_all (source->priv->chooser);
}

#ifdef BUILD_SEARCH
static void
bonfire_file_source_show_search (BonfireFileSource *source)
{
	GtkWidget *label;

	source->priv->search = bonfire_search_new ();
	g_signal_connect (G_OBJECT (source->priv->search), "activated",
			  G_CALLBACK (bonfire_file_source_search_activated_cb),
			  source);
	g_signal_connect (G_OBJECT (source->priv->search),
			  "selection_changed",
			  G_CALLBACK (bonfire_file_source_search_selection_changed_cb),
			  source);
	label = gtk_label_new (_("Search"));
	gtk_notebook_insert_page (GTK_NOTEBOOK (source->priv->notebook),
				  source->priv->search,
				  label,
				  1);
	gtk_widget_show_all (source->priv->search);
}
#endif

#ifdef BUILD_PLAYLIST
static void
bonfire_file_source_show_playlist (BonfireFileSource *source)
{
	GtkWidget *label;

	source->priv->playlist = bonfire_playlist_new ();
	g_signal_connect (G_OBJECT (source->priv->playlist),
			  "activated",
			  G_CALLBACK (bonfire_file_source_playlist_activated_cb),
			  source);
	g_signal_connect (G_OBJECT (source->priv->playlist),
			  "selection_changed",
			  G_CALLBACK (bonfire_file_source_playlist_selection_changed_cb),
			  source);
	label = gtk_label_new (_("Playlists"));
	gtk_notebook_insert_page (GTK_NOTEBOOK (source->priv->notebook),
				  source->priv->playlist,
				  label,
				  2);
	gtk_widget_show_all (source->priv->playlist);
}
#endif

#ifdef BUILD_PREVIEW
static void
bonfire_file_source_show_viewer (BonfireFileSource *source)
{
	source->priv->viewer = bonfire_player_new ();
	gtk_paned_pack2 (GTK_PANED (source), source->priv->viewer, TRUE, FALSE);
}
#endif

static void
bonfire_file_source_finalize (GObject * object)
{
	BonfireFileSource *cobj;

	cobj = BONFIRE_FILE_SOURCE (object);

	/* Free private members, etc. */
	gconf_client_remove_dir (cobj->priv->client, BONFIRE_DISPLAY_DIR,
				 NULL);

	gconf_client_notify_remove (cobj->priv->client,
				    cobj->priv->chooser_notify);

#ifdef BUILD_PREVIEW
	gconf_client_notify_remove (cobj->priv->client,
				    cobj->priv->viewer_notify);
#endif

#ifdef BUILD_SEARCH
	gconf_client_notify_remove (cobj->priv->client,
				    cobj->priv->search_notify);
#endif

#ifdef BUILD_PLAYLIST
	gconf_client_notify_remove (cobj->priv->client,
				    cobj->priv->playlist_notify);
#endif

	g_object_unref (cobj->priv->client);

	g_free (cobj->priv);
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

GtkWidget *
bonfire_file_source_new ()
{
	BonfireFileSource *obj;

	obj = BONFIRE_FILE_SOURCE (g_object_new (BONFIRE_TYPE_FILE_SOURCE, NULL));

	return GTK_WIDGET (obj);
}

void
bonfire_file_source_show (BonfireFileSource * source)
{
	gtk_widget_show (GTK_WIDGET (source));
	gtk_action_group_set_sensitive (source->priv->action_group, TRUE);
}

void
bonfire_file_source_hide (BonfireFileSource * source)
{
	gtk_widget_hide (GTK_WIDGET (source));
	gtk_action_group_set_sensitive (source->priv->action_group, FALSE);
}

void
bonfire_file_source_add_menu (BonfireFileSource * source,
			      GtkUIManager * ui_manager)
{
	GError *error = NULL;

	gtk_ui_manager_insert_action_group (ui_manager,
					    source->priv->action_group, 0);

	if (!gtk_ui_manager_add_ui_from_string (ui_manager, description, -1, &error)) {
		g_message ("building menus failed: %s", error->message);
		g_error_free (error);
	}
}

char **
bonfire_file_source_get_uris (BonfireFileSource * source)
{
	char **uris;
	gint current;
	GtkWidget *widget;

	current = gtk_notebook_get_current_page (GTK_NOTEBOOK (source->priv->notebook));
	widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK (source->priv->notebook),
					    current);

	if (!widget)
		return NULL;

	if (!GTK_WIDGET_MAPPED (widget)
	||  !GTK_WIDGET_VISIBLE (widget))
		return NULL;

	if (widget == source->priv->chooser) {
		GSList *list, *iter;
		int i;

		list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (source->priv->chooser));
		if (list == NULL)
			return NULL;

		uris = g_new0 (char *, g_slist_length (list) + 1);
		for (iter = list, i = 0; iter; iter = iter->next, i++)
			uris [i] = (char *) iter->data;
		g_slist_free (list);
	}
#ifdef BUILD_SEARCH
	else if (widget == source->priv->search)
		uris = bonfire_search_get_selected_uris (BONFIRE_SEARCH (source->priv->search));
#endif

#ifdef BUILD_PLAYLIST
	else if (widget == source->priv->playlist)
		uris = bonfire_playlist_get_selected_uris (BONFIRE_PLAYLIST (source->priv->playlist));
#endif
	else
		uris = NULL;

	return uris;
}

static void
bonfire_file_source_selection_changed (BonfireFileSource *source)
{
	gboolean value;
	int page_num;

	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (source->priv->notebook));
	if (page_num != -1)
		value = FALSE;
	else
		value = TRUE;

	g_signal_emit (source,
		       bonfire_file_source_signals [SELECTION_CHANGED_SIGNAL],
		       0, value);

#ifdef BUILD_PREVIEW
	char **uris;

	if (!source->priv->viewer)
		return;

	uris = bonfire_file_source_get_uris (source);
	if (uris) {
		const char *uri;

		uri = *uris;
		bonfire_player_set_uri (BONFIRE_PLAYER (source->priv->viewer), uri);
		g_strfreev (uris);
	}
	else
		bonfire_player_set_uri (BONFIRE_PLAYER (source->priv->viewer), NULL);
#endif
}

static void
bonfire_file_source_switch_page_cb (GtkNotebook * notebook,
				    GtkNotebookPage * page,
				    guint page_num,
				    BonfireFileSource * source)
{
	bonfire_file_source_selection_changed (source);
}

static void
bonfire_file_source_chooser_toggled_cb (GtkToggleAction * action,
					BonfireFileSource * source)
{
	if (gtk_toggle_action_get_active (action) == FALSE) {
		if (source->priv->chooser) {
			gtk_widget_destroy (source->priv->chooser);
			source->priv->chooser = NULL;

			bonfire_file_source_selection_changed (source);
		}
	}
	else if (!source->priv->chooser)
		bonfire_file_source_show_chooser (source);

	gconf_client_set_bool (source->priv->client,
			       BONFIRE_KEY_DISPLAY_CHOOSER,
			       gtk_toggle_action_get_active (action),
			       NULL);
}

static void
bonfire_file_source_chooser_activated_cb (GtkFileChooser * chooser,
					  BonfireFileSource * source)
{
	g_signal_emit (source,
		       bonfire_file_source_signals[ACTIVATED_SIGNAL], 0);
}

static void
bonfire_file_source_chooser_selection_changed_cb (GtkFileChooser * chooser,
						  BonfireFileSource * source)
{
	bonfire_file_source_selection_changed (source);
}

static void
bonfire_file_source_display_chooser_changed_cb (GConfClient * client,
						guint cxn,
						GConfEntry * entry,
						BonfireFileSource * source)
{
	GConfValue *value;
	GtkToggleAction *action;

	value = gconf_entry_get_value (entry);
	if (value->type != GCONF_VALUE_BOOL)
		return;

	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
				    (source->priv->action_group, "Chooser"));

	if (gconf_value_get_bool (value) == TRUE) {
		if (!gtk_toggle_action_get_active (action)) {
			bonfire_file_source_show_chooser (source);
			gtk_toggle_action_set_active (action, TRUE);
		}
	}
	else if (gtk_toggle_action_get_active (action)) {
		gtk_widget_destroy (source->priv->chooser);
		source->priv->chooser = NULL;
		bonfire_file_source_selection_changed (source);

		gtk_toggle_action_set_active (action, FALSE);
	}

	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (source->priv->notebook)) > 1)
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (source->priv->notebook),
					    TRUE);
	else
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (source->priv->notebook),
					    FALSE);
}

#ifdef BUILD_SEARCH
static void
bonfire_file_source_search_toggled_cb (GtkToggleAction * action,
				       BonfireFileSource * source)
{
	if (gtk_toggle_action_get_active (action) == FALSE) {
		if (source->priv->search) {
			gtk_widget_destroy (source->priv->search);
			source->priv->search = NULL;
			bonfire_file_source_selection_changed (source);
		}
	}
	else if (!source->priv->search)
		bonfire_file_source_show_search (source);

	gconf_client_set_bool (source->priv->client,
			       BONFIRE_KEY_DISPLAY_SEARCH,
			       gtk_toggle_action_get_active (action),
			       NULL);
}

static void
bonfire_file_source_search_activated_cb (BonfireSearch * search,
					 BonfireFileSource * source)
{
	g_signal_emit (source,
		       bonfire_file_source_signals[ACTIVATED_SIGNAL], 0);
}

static void
bonfire_file_source_search_selection_changed_cb (BonfireSearch * search,
						 BonfireFileSource * source)
{
	bonfire_file_source_selection_changed (source);
}

static void
bonfire_file_source_display_search_changed_cb (GConfClient * client,
					       guint cxn,
					       GConfEntry * entry,
					       BonfireFileSource * source)
{
	GConfValue *value;
	GtkToggleAction *action;

	value = gconf_entry_get_value (entry);
	if (value->type != GCONF_VALUE_BOOL)
		return;

	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
				    (source->priv->action_group, "Search"));

	if (gconf_value_get_bool (value)) {
		if (!gtk_toggle_action_get_active (action)) {
			bonfire_file_source_show_search (source);
			gtk_toggle_action_set_active (action, TRUE);
		}
	}
	else if (gtk_toggle_action_get_active (action)) {
		gtk_widget_destroy (source->priv->search);
		source->priv->search = NULL;
		bonfire_file_source_selection_changed (source);

		gtk_toggle_action_set_active (action, FALSE);
	}

	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (source->priv->notebook)) > 1)
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (source->priv->notebook),
					    TRUE);
	else
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (source->priv->notebook),
					    FALSE);
}
#endif

#ifdef BUILD_PLAYLIST
static void
bonfire_file_source_playlist_toggled_cb (GtkToggleAction * action,
					 BonfireFileSource * source)
{
	if (gtk_toggle_action_get_active (action) == FALSE) {
		if (source->priv->playlist) {
			gtk_widget_destroy (source->priv->playlist);
			source->priv->playlist = NULL;
			bonfire_file_source_selection_changed (source);
		}
	}
	else if (!source->priv->playlist)
		bonfire_file_source_show_playlist (source);

	gconf_client_set_bool (source->priv->client,
			       BONFIRE_KEY_DISPLAY_PLAYLIST,
			       gtk_toggle_action_get_active (action),
			       NULL);
}

static void
bonfire_file_source_playlist_activated_cb (BonfirePlaylist * playlist,
					   BonfireFileSource * source)
{
	g_signal_emit (source,
		       bonfire_file_source_signals[ACTIVATED_SIGNAL], 0);
}

static void
bonfire_file_source_playlist_selection_changed_cb (BonfirePlaylist *
						   playlist,
						   BonfireFileSource *
						   source)
{
	bonfire_file_source_selection_changed (source);
}

static void
bonfire_file_source_display_playlist_changed_cb (GConfClient *client,
						 guint cxn,
						 GConfEntry *entry,
						 BonfireFileSource *source)
{
	GConfValue *value;
	GtkToggleAction *action;

	value = gconf_entry_get_value (entry);
	if (value->type != GCONF_VALUE_BOOL)
		return;

	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
				    (source->priv->action_group, "Playlist"));

	if (gconf_value_get_bool (value)) {
		if (!gtk_toggle_action_get_active (action)) {
			bonfire_file_source_show_playlist (source);
			gtk_toggle_action_set_active (action, TRUE);
		}
	}
	else if (gtk_toggle_action_get_active (action)) {
		gtk_widget_destroy (source->priv->playlist);
		source->priv->playlist = NULL;
		bonfire_file_source_selection_changed (source);

		gtk_toggle_action_set_active (action, FALSE);
	}

	if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (source->priv->notebook)) > 1)
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (source->priv->notebook),
					    TRUE);
	else
		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (source->priv->notebook),
					    FALSE);
}
#endif

#ifdef BUILD_PREVIEW
static void
bonfire_file_source_viewer_toggled_cb (GtkToggleAction *action,
				       BonfireFileSource *source)
{
	if (gtk_toggle_action_get_active (action) == TRUE)
		gtk_widget_show_all (source->priv->viewer);
	else
		gtk_widget_hide (source->priv->viewer);

	gconf_client_set_bool (source->priv->client,
			       BONFIRE_KEY_DISPLAY_VIEWER,
			       gtk_toggle_action_get_active (action),
			       NULL);
}

static void
bonfire_file_source_display_viewer_changed_cb (GConfClient *client,
					       guint cxn,
					       GConfEntry *entry,
					       BonfireFileSource *source)
{
	GConfValue *value;
	GtkToggleAction *action;

	value = gconf_entry_get_value (entry);
	if (value->type != GCONF_VALUE_BOOL)
		return;

	action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
				    (source->priv->action_group, "Viewer"));

	if (gconf_value_get_bool (value) == TRUE) {
		if (!gtk_toggle_action_get_active (action)) {
			gtk_toggle_action_set_active (action, TRUE);
			gtk_widget_show_all (source->priv->viewer);
		}
	}
	else if (gtk_toggle_action_get_active (action)) {
		gtk_toggle_action_set_active (action, FALSE);
		gtk_widget_hide (source->priv->viewer);
	}
}
#endif

#endif				/* BUILD_GDL */
